"use strict";
/*
 * Licensed to Elasticsearch B.V. under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. Elasticsearch B.V. licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("@kbn/i18n/react");
const classnames_1 = __importDefault(require("classnames"));
const react_2 = __importStar(require("react"));
const resize_observer_polyfill_1 = __importDefault(require("resize-observer-polyfill"));
const lodash_1 = require("lodash");
const notify_1 = require("ui/notify");
const data_1 = require("plugins/data");
const save_query_form_1 = require("plugins/data/search/search_bar/components/saved_query_management/save_query_form");
const saved_query_management_component_1 = require("plugins/data/search/search_bar/components/saved_query_management/saved_query_management_component");
const saved_query_service_1 = require("plugins/data/search/search_bar/lib/saved_query_service");
const filter_bar_1 = require("./filter_bar");
class SearchBarUI extends react_2.Component {
    constructor() {
        super(...arguments);
        this.filterBarRef = null;
        this.filterBarWrapperRef = null;
        /*
         Keep the "draft" value in local state until the user actually submits the query. There are a couple advantages:
      
          1. Each app doesn't have to maintain its own "draft" value if it wants to put off updating the query in app state
          until the user manually submits their changes. Most apps have watches on the query value in app state so we don't
          want to trigger those on every keypress. Also, some apps (e.g. dashboard) already juggle multiple query values,
          each with slightly different semantics and I'd rather not add yet another variable to the mix.
      
          2. Changes to the local component state won't trigger an Angular digest cycle. Triggering digest cycles on every
          keypress has been a major source of performance issues for us in previous implementations of the query bar.
          See https://github.com/elastic/kibana/issues/14086
        */
        this.state = {
            isFiltersVisible: true,
            showSaveQueryModal: false,
            showSaveNewQueryModal: false,
            showSavedQueryPopover: false,
            currentProps: this.props,
            query: this.props.query ? { ...this.props.query } : undefined,
            dateRangeFrom: lodash_1.get(this.props, 'dateRangeFrom', 'now-15m'),
            dateRangeTo: lodash_1.get(this.props, 'dateRangeTo', 'now'),
            savedQueryService: saved_query_service_1.createSavedQueryService(this.props.savedObjectsClient),
        };
        this.isDirty = () => {
            if (!this.props.showDatePicker && this.state.query && this.props.query) {
                return this.state.query.query !== this.props.query.query;
            }
            return ((this.state.query && this.props.query && this.state.query.query !== this.props.query.query) ||
                this.state.dateRangeFrom !== this.props.dateRangeFrom ||
                this.state.dateRangeTo !== this.props.dateRangeTo);
        };
        this.setFilterBarHeight = () => {
            requestAnimationFrame(() => {
                const height = this.filterBarRef && this.state.isFiltersVisible ? this.filterBarRef.clientHeight : 0;
                if (this.filterBarWrapperRef) {
                    this.filterBarWrapperRef.setAttribute('style', `height: ${height}px`);
                }
            });
        };
        // member-ordering rules conflict with use-before-declaration rules
        /* eslint-disable */
        this.ro = new resize_observer_polyfill_1.default(this.setFilterBarHeight);
        /* eslint-enable */
        this.onSave = async (savedQueryMeta, saveAsNew = false) => {
            if (!this.state.query)
                return;
            const savedQueryAttributes = {
                title: savedQueryMeta.title,
                description: savedQueryMeta.description,
                query: this.state.query,
            };
            if (savedQueryMeta.shouldIncludeFilters) {
                savedQueryAttributes.filters = this.props.filters;
            }
            if (savedQueryMeta.shouldIncludeTimefilter &&
                this.state.dateRangeTo !== undefined &&
                this.state.dateRangeFrom !== undefined &&
                this.props.refreshInterval !== undefined &&
                this.props.isRefreshPaused !== undefined) {
                savedQueryAttributes.timefilter = {
                    from: this.state.dateRangeFrom,
                    to: this.state.dateRangeTo,
                    refreshInterval: {
                        value: this.props.refreshInterval,
                        pause: this.props.isRefreshPaused,
                    },
                };
            }
            try {
                let response;
                if (this.props.savedQuery && !saveAsNew) {
                    response = await this.state.savedQueryService.saveQuery(savedQueryAttributes, {
                        overwrite: true,
                    });
                }
                else {
                    response = await this.state.savedQueryService.saveQuery(savedQueryAttributes);
                }
                notify_1.toastNotifications.addSuccess(`Your query "${response.attributes.title}" was saved`);
                this.setState({
                    showSaveQueryModal: false,
                    showSaveNewQueryModal: false,
                });
                if (this.props.onSaved) {
                    this.props.onSaved(response);
                }
                if (this.props.onQuerySubmit) {
                    this.props.onQuerySubmit({
                        query: this.state.query,
                        dateRange: {
                            from: this.state.dateRangeFrom,
                            to: this.state.dateRangeTo,
                        },
                    });
                }
            }
            catch (error) {
                notify_1.toastNotifications.addDanger(`An error occured while saving your query: ${error.message}`);
                throw error;
            }
        };
        this.onInitiateSave = () => {
            this.setState({
                showSaveQueryModal: true,
            });
        };
        this.onInitiateSaveNew = () => {
            this.setState({
                showSaveNewQueryModal: true,
            });
        };
        this.onQueryBarChange = (queryAndDateRange) => {
            this.setState({
                query: queryAndDateRange.query,
                dateRangeFrom: queryAndDateRange.dateRange.from,
                dateRangeTo: queryAndDateRange.dateRange.to,
            });
        };
        this.onQueryBarSubmit = (queryAndDateRange) => {
            this.setState({
                query: queryAndDateRange.query,
                dateRangeFrom: (queryAndDateRange.dateRange && queryAndDateRange.dateRange.from) ||
                    this.state.dateRangeFrom,
                dateRangeTo: (queryAndDateRange.dateRange && queryAndDateRange.dateRange.to) || this.state.dateRangeTo,
            }, () => {
                if (this.props.onQuerySubmit) {
                    this.props.onQuerySubmit({
                        query: this.state.query,
                        dateRange: {
                            from: this.state.dateRangeFrom,
                            to: this.state.dateRangeTo,
                        },
                    });
                }
            });
        };
        this.onLoadSavedQuery = (savedQuery) => {
            const dateRangeFrom = lodash_1.get(savedQuery, 'attributes.timefilter.from', this.state.dateRangeFrom);
            const dateRangeTo = lodash_1.get(savedQuery, 'attributes.timefilter.to', this.state.dateRangeTo);
            this.setState({
                query: savedQuery.attributes.query,
                dateRangeFrom,
                dateRangeTo,
            });
            if (this.props.onSavedQueryUpdated) {
                this.props.onSavedQueryUpdated(savedQuery);
            }
        };
    }
    static getDerivedStateFromProps(nextProps, prevState) {
        if (lodash_1.isEqual(prevState.currentProps, nextProps)) {
            return null;
        }
        let nextQuery = null;
        if (nextProps.query && nextProps.query.query !== lodash_1.get(prevState, 'currentProps.query.query')) {
            nextQuery = {
                query: nextProps.query.query,
                language: nextProps.query.language,
            };
        }
        else if (nextProps.query &&
            prevState.query &&
            nextProps.query.language !== prevState.query.language) {
            nextQuery = {
                query: '',
                language: nextProps.query.language,
            };
        }
        let nextDateRange = null;
        if (nextProps.dateRangeFrom !== lodash_1.get(prevState, 'currentProps.dateRangeFrom') ||
            nextProps.dateRangeTo !== lodash_1.get(prevState, 'currentProps.dateRangeTo')) {
            nextDateRange = {
                dateRangeFrom: nextProps.dateRangeFrom,
                dateRangeTo: nextProps.dateRangeTo,
            };
        }
        const nextState = {
            currentProps: nextProps,
        };
        if (nextQuery) {
            nextState.query = nextQuery;
        }
        if (nextDateRange) {
            nextState.dateRangeFrom = nextDateRange.dateRangeFrom;
            nextState.dateRangeTo = nextDateRange.dateRangeTo;
        }
        return nextState;
    }
    getFilterUpdateFunction() {
        if (this.props.showFilterBar && this.props.onFiltersUpdated) {
            return this.props.onFiltersUpdated;
        }
        return (filters) => { };
    }
    shouldRenderQueryBar() {
        const showDatePicker = this.props.showDatePicker || this.props.showAutoRefreshOnly;
        const showQueryInput = this.props.showQueryInput && this.props.indexPatterns && this.state.query;
        return this.props.showQueryBar && (showDatePicker || showQueryInput);
    }
    shouldRenderFilterBar() {
        return this.props.showFilterBar && this.props.filters && this.props.indexPatterns;
    }
    componentDidMount() {
        if (this.filterBarRef) {
            this.setFilterBarHeight();
            this.ro.observe(this.filterBarRef);
        }
    }
    componentDidUpdate() {
        if (this.filterBarRef) {
            this.setFilterBarHeight();
            this.ro.unobserve(this.filterBarRef);
        }
    }
    render() {
        // This is needed, as kbn-top-nav might render before npSetup.core.uiSettings is set.
        // This won't be needed when it's loaded exclusively with React.
        if (!this.props.uiSettings) {
            return null;
        }
        const savedQueryManagement = this.state.query && this.props.onClearSavedQuery && (react_2.default.createElement(saved_query_management_component_1.SavedQueryManagementComponent, { showSaveQuery: this.props.showSaveQuery, loadedSavedQuery: this.props.savedQuery, onSave: this.onInitiateSave, onSaveAsNew: this.onInitiateSaveNew, onLoad: this.onLoadSavedQuery, savedQueryService: this.state.savedQueryService, onClearSavedQuery: this.props.onClearSavedQuery }));
        let queryBar;
        if (this.shouldRenderQueryBar()) {
            queryBar = (react_2.default.createElement(data_1.QueryBar, { toasts: this.props.toasts, uiSettings: this.props.uiSettings, savedObjectsClient: this.props.savedObjectsClient, query: this.state.query, screenTitle: this.props.screenTitle, onSubmit: this.onQueryBarSubmit, appName: this.props.appName, indexPatterns: this.props.indexPatterns, store: this.props.store, prepend: this.props.showFilterBar ? savedQueryManagement : undefined, showDatePicker: this.props.showDatePicker, dateRangeFrom: this.state.dateRangeFrom, dateRangeTo: this.state.dateRangeTo, isRefreshPaused: this.props.isRefreshPaused, refreshInterval: this.props.refreshInterval, showAutoRefreshOnly: this.props.showAutoRefreshOnly, showQueryInput: this.props.showQueryInput, onRefreshChange: this.props.onRefreshChange, onChange: this.onQueryBarChange, isDirty: this.isDirty(), customSubmitButton: this.props.customSubmitButton ? this.props.customSubmitButton : undefined }));
        }
        let filterBar;
        if (this.shouldRenderFilterBar()) {
            const filterGroupClasses = classnames_1.default('globalFilterGroup__wrapper', {
                'globalFilterGroup__wrapper-isVisible': this.state.isFiltersVisible,
            });
            filterBar = (react_2.default.createElement("div", { id: "GlobalFilterGroup", ref: node => {
                    this.filterBarWrapperRef = node;
                }, className: filterGroupClasses },
                react_2.default.createElement("div", { ref: node => {
                        this.filterBarRef = node;
                    } },
                    react_2.default.createElement(filter_bar_1.FilterBar, { className: "globalFilterGroup__filterBar", uiSettings: this.props.uiSettings, filters: this.props.filters, onFiltersUpdated: this.getFilterUpdateFunction(), indexPatterns: this.props.indexPatterns }))));
        }
        return (react_2.default.createElement("div", { className: "globalQueryBar" },
            queryBar,
            filterBar,
            this.state.showSaveQueryModal ? (react_2.default.createElement(save_query_form_1.SaveQueryForm, { savedQuery: this.props.savedQuery ? this.props.savedQuery.attributes : undefined, savedQueryService: this.state.savedQueryService, onSave: this.onSave, onClose: () => this.setState({ showSaveQueryModal: false }), showFilterOption: this.props.showFilterBar, showTimeFilterOption: this.props.showDatePicker })) : null,
            this.state.showSaveNewQueryModal ? (react_2.default.createElement(save_query_form_1.SaveQueryForm, { savedQueryService: this.state.savedQueryService, onSave: savedQueryMeta => this.onSave(savedQueryMeta, true), onClose: () => this.setState({ showSaveNewQueryModal: false }), showFilterOption: this.props.showFilterBar, showTimeFilterOption: this.props.showDatePicker })) : null));
    }
}
SearchBarUI.defaultProps = {
    showQueryBar: true,
    showFilterBar: true,
    showDatePicker: true,
    showAutoRefreshOnly: false,
};
exports.SearchBar = react_1.injectI18n(SearchBarUI);
